/*********************************************************************
 * This file is part of the PRAPI library.
 *
 * Copyright (C) 2001 Topi Menp
 * All rights reserved.
 *
 * This program is free software. You can redistribute and/or modify
 * it under the terms of the free software licence found in the
 * accompanying file "COPYING". The licence terms must always be
 * redistributed with this source file. The above copyright notice
 * must be reproduced in all modified and unmodified copies of this
 * source file.
 *
 * $Revision: 1.2 $
 *********************************************************************/

#ifndef _COLORMAP_H
#define _COLORMAP_H

#include <List.h>
#include "Color.h"

using namespace util;

namespace prapi
{
	/**
	 * ColorMap is a convenience class for creating color maps. It is
	 * just a list of RGBColor objects and it is used by some image
	 * codecs.
	 **/
	template <class T>
	class ColorMap : public List< RGBColor<T> >
	{
	public:
		ColorMap(int initialSlots = 16, int blockSize = 16) : List< RGBColor<T> >(initialSlots, blockSize) {}
		ColorMap(const ColorMap& other) : List<RGBColor<T> >(other) {}
		ColorMap(const List< RGBColor<T> >& other) : List< RGBColor<T> >(other) {}

		ColorMap& operator= (const ColorMap& other) { List<RGBColor<T> >::operator= (other); return *this; }

		/**
		 * Generate a map of gray values.
		 * @param levels the number of gray levels
		 * @param min lowest gray level
		 * @param max highest gray level 
		 **/
		void generateGrayMap(int levels = 256, T min = 0, T max = 256);

		/**
		 * Get one channel from the color map as a list.
		 * @param index the channel index. 0 = red, 1 = green, 2 = blue
		 **/
		List<T> getChannel(int index);
	};

	template <class T>
	List<T> ColorMap<T>::getChannel(int index)
	{
		List<T> result(_iCurrentItems);
		result.setLength(_iCurrentItems);
		for (int i=0;i<_iCurrentItems;i++)
			{
				result[i] = _internalArray[i][index];
				//cerr << result[i] << " ";
			}
		return result;
	}

	template <class T>
	void ColorMap<T>::generateGrayMap(int levels, T min, T max)
	{
		clear();
		setCapacity(levels);
		for (int i=0;i<levels;i++)
			{
				T value = min + i*(max-min)/levels;

				RGBColor<T> clr(value,value,value);
				addElement(clr);
			}
	}
}

#endif
