/*********************************************************************
 * This file is part of the PRAPI library.
 *
 * Copyright (C) 2001 Topi Menp and Jaakko Viertola
 * All rights reserved.
 *
 * This program is free software. You can redistribute and/or modify
 * it under the terms of the free software licence found in the
 * accompanying file "COPYING". The licence terms must always be
 * redistributed with this source file. The above copyright notice
 * must be reproduced in all modified and unmodified copies of this
 * source file.
 *
 * $Revision: 1.5 $
 *********************************************************************/

#include "ColorCorrelogram.h"

namespace prapi { namespace color {
	
	List<int> ColorCorrelogram::getFeatureVector(const RGBColorImage<>& img)
		throw (FeatureExtractionException&)
	{
		Matrix<int> indexed(flatten(img,_iLevels,_iOriginalLevels));
		
		List<int> result(_iLevels*_lstDistances.getLength());
		result.setLength(_iLevels*_lstDistances.getLength());
		result = 0;
		int cols = indexed.getColumns();
		
		for (int i=_lstDistances.getLength();i--;)
			{
				int *ptrs[_lstDistances[i]<<3];
				int dLength = _lstDistances[i]<<1;
				ptrs[0] = indexed.getData();
				int *previous = ptrs[0], *current = previous+1;
				for (int j=dLength;j--;previous++, current++)
					*current = *previous + 1;
				for (int j=dLength;j--;previous++, current++)
					*current = *previous + cols;
				for (int j=dLength;j--;previous++, current++)
					*current = *previous - 1;
				for (int j=dLength;j-->1;previous++, current++)
					*current = *previous - cols;
				
				int* data = indexed.getData() + (_lstDistances[i] + 1) * cols;
				int start = _iLevels * i;
				for (int r=indexed.getRows();r--;)
					{
						for (int c=indexed.getColumns();c--;data++)
							{
								int cnt = 0;
								for (int k=_lstDistances[i]<<3;k--;)
									{
										if (*ptrs[k] == *data)
											cnt++;
										ptrs[k]++;
									}
								result[start+*data] += cnt;
							}
						for (int k=_lstDistances[i]<<3;k--;)
							ptrs[k] += dLength;
						data += dLength;
					}
			}
	 		return result; 
	}
}}
