/*********************************************************************
 * This file is part of the PRAPI library.
 *
 * Copyright (C) 2001 Topi Menp and Jaakko Viertola
 * All rights reserved.
 *
 * This program is free software. You can redistribute and/or modify
 * it under the terms of the free software licence found in the
 * accompanying file "COPYING". The licence terms must always be
 * redistributed with this source file. The above copyright notice
 * must be reproduced in all modified and unmodified copies of this
 * source file.
 *
 * $Revision: 1.6 $
 *********************************************************************/

#ifndef _COLORCORRELOGRAM_H
#define _COLORCORRELOGRAM_H

#include <Matrix.h>
#include <List.h>

#include "../RGBColorImage.h"
#include "../Color.h"
#include "../FeatureExtractor.h"
#include "../Indexer.h"

using namespace util;

namespace prapi { namespace color {
	
	/**
	 * A feature extractor that calculates the color correlogram for an
	 * RGB color image. The method is described in detail in <i>Jing
	 * Huang; Kumar, S.R.; Mitra, M.; Wei-Jing Zhu; Zabih, R.: "Image
	 * indexing using color correlograms", Computer Vision and Pattern
	 * Recognition, 1997. Proceedings., 1997 IEEE Computer Society
	 * Conference on, 1997, Pages: 762 -768</i>
	 **/
	class ColorCorrelogram : public FeatureExtractor<int, RGBColorImage<> >, public Indexer
	{
	public:
		/**
		 * Create a color correlogram extractor with the given parameters.
		 *
		 * @param levels the number of quantization levels for a color channel
		 * @param originalLevels the original number of quantization levels
		 * @param distances the distances on which the correlogram is calculated
		 **/
		ColorCorrelogram(int levels, int originalLevels, const List<int>& distances) :
			_lstDistances(distances), _iLevels(levels), _iOriginalLevels(levels) {}
		
		List<int> getFeatureVector(const RGBColorImage<>& img) throw (FeatureExtractionException&);
		
	private:
		List<int> _lstDistances;
		int _iLevels, _iOriginalLevels;
	};
}}
#endif
