/*********************************************************************
 * This file is part of the PRAPI library.
 *
 * Copyright (C) 2001 Topi Menp and Jaakko Viertola
 * All rights reserved.
 *
 * This program is free software. You can redistribute and/or modify
 * it under the terms of the free software licence found in the
 * accompanying file "COPYING". The licence terms must always be
 * redistributed with this source file. The above copyright notice
 * must be reproduced in all modified and unmodified copies of this
 * source file.
 *
 * $Revision: 1.6 $
 *********************************************************************/

#include "ColorPercentile.h"

namespace prapi { namespace color {

  ColorPercentile::ColorPercentile() : _dlstPercentiles(10)
	{
		for (double i=0.1;i<=1.0;i+=0.1)
			_dlstPercentiles += i;
	}

	int ColorPercentile::getPercentile(double percentile, List<int> histogram, int total)
	{
		int sum = 0;
		double tot = total > 0 ? total : Math::sum(histogram);
		for (int i=0;i<histogram.getLength();i++)
			{
				sum += histogram[i];
				if (double(sum)/tot >= percentile)
					return i;
			}
		return histogram.getLength();
	}
	
	List<int> ColorPercentile::getFeatureVector(const Matrix<int>& mat)
		throw (FeatureExtractionException&)
	{
		IntegerHistogram hst;
		List<int> histogram(hst.getFeatureVector(mat));
		int total = Math::sum(histogram);

		List<int> result(_dlstPercentiles.getLength());

		for (int i=0;i<_dlstPercentiles.getLength();i++)
			result += getPercentile(_dlstPercentiles[i],histogram,total);
		
		return result;
	}
}}
