/*********************************************************************
 * This file is part of the PRAPI library.
 *
 * Copyright (C) 2001 Topi Menp
 * All rights reserved.
 *
 * This program is free software. You can redistribute and/or modify
 * it under the terms of the free software licence found in the
 * accompanying file "COPYING". The licence terms must always be
 * redistributed with this source file. The above copyright notice
 * must be reproduced in all modified and unmodified copies of this
 * source file.
 *
 * $Revision: 1.1 $
 *********************************************************************/

#ifndef _COLORRATIO_H
#define _COLORRATIO_H

#include "../Color.h"
#include "../ImageTransform.h"

#include <texture/EdgeDetector.h>

namespace prapi { namespace color {
	/**
	 * ColorRatio implements the preprocessing step of the color constant
	 * color indexing method of Funt and Finlayson. It calculates the LoG
	 * operator for the logarithm of an input image.
	 **/
	template <class T=int> class ColorRatio : public ImageTransform<Color<double,3>, Color<T,3> >
	{
	public:
		util::Matrix<Color<double,3> > getTransformedImage(const util::Matrix<Color<T,3> >& mat)
			throw (ImageTransformException&);
	};

	template <class T> util::Matrix<Color<double,3> > ColorRatio<T>::getTransformedImage(const util::Matrix<Color<T,3> >& mat)
		throw (ImageTransformException&)
	{
		using namespace std;
		prapi::texture::LaplaceOfGaussian<Color<double,3>, Color<double,3> > laplacian;
		util::Matrix<Color<double,3> > logarithm(mat.getRows(),mat.getColumns());

		Color<double,3>* logData = logarithm.getData();
		const Color<T,3>* sourceData = mat.getData();
	
		for (int i=mat.getRows()*mat.getColumns(); i--; logData++, sourceData++)
			{
				(*logData)[0] = log((double)(*sourceData)[0]);
				(*logData)[1] = log((double)(*sourceData)[1]);
				(*logData)[2] = log((double)(*sourceData)[2]);
			}
	
		return laplacian.getTransformedImage(logarithm);
	}
}}

#endif
