/*********************************************************************
 * This file is part of the PRAPI library.
 *
 * Copyright (C) 2001 Topi Menp and Jaakko Viertola
 * All rights reserved.
 *
 * This program is free software. You can redistribute and/or modify
 * it under the terms of the free software licence found in the
 * accompanying file "COPYING". The licence terms must always be
 * redistributed with this source file. The above copyright notice
 * must be reproduced in all modified and unmodified copies of this
 * source file.
 *
 * $Revision: 1.3 $
 *********************************************************************/

#include "CompNorm.h"

using namespace util;

namespace prapi { namespace color {

	Matrix<RGBColor<> > CompNorm::getTransformedImage(const Matrix<RGBColor<> >& mat) throw (ImageTransformException&)
	{
		Matrix<RGBColor<double> > img(mat);
		int r = img.getRows(), c = img.getColumns(), size=r*c;

		for (int i=0;i<5;i++)
			{
				//Normalize against illumination geometry changes
				RGBColor<double>* ptr = img.getData();
				for (int j=size;j--;ptr++)
					*ptr = ptr->getNormalizedColor();

				//Normalize against illumination color changes
				RGBColor<double> sum = img.sum() / ((double)size / 3.0);
				ptr = img.getData();
				for (int j=size;j--;ptr++)
					*ptr = (*ptr)/sum;
			}

		Matrix<RGBColor<> > result(r,c);
		RGBColor<> *tptr = result.getData();
		RGBColor<double> *sptr = img.getData();
		for (int i=size;i--;sptr++,tptr++)
			*tptr = RGBColor<>((*sptr) * 255.0 + 0.5);
			
		return result;
	}

}}
